@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                            )--------------------
-# Copyright (c) 2014-2023 Beartype authors.
+# Copyright (c) 2014-2025 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -14,9 +14,17 @@ This private submodule is *not* intended for importation by downstream callers.
 # ....................{ IMPORTS                            }....................
 from beartype.typing import (
     Any,
+    List,
     Tuple,
 )
 
+# ....................{ LISTS                              }....................
+# Note that this exact type annotation is required to avoid mypy complaints. :O
+LIST_EMPTY: List[Any] = []
+'''
+**Empty list singleton.**
+'''
+
 # ....................{ TUPLES                             }....................
 # Note that this exact type annotation is required to avoid mypy complaints. :O
 TUPLE_EMPTY: Tuple[Any, ...] = ()
@@ -24,8 +32,9 @@ TUPLE_EMPTY: Tuple[Any, ...] = ()
 **Empty tuple singleton.**
 
 Yes, we know exactly what you're thinking: "Why would anyone do this, @leycec?
-Why not just directly access the empty tuple singleton as ()?" Because Python
-insanely requires us to do this under Python >= 3.8 to detect empty tuples:
+Why not just directly access the empty tuple singleton as ``()``?" Because
+Python insanely requires us to do this under Python >= 3.8 to detect empty
+tuples:
 
 .. code-block:: bash
 
