@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                            )--------------------
-# Copyright (c) 2014-2023 Beartype authors.
+# Copyright (c) 2014-2025 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -18,13 +18,14 @@ This private submodule is *not* intended for importation by downstream callers.
 #    from beartype._util.module.utilmodget import die_unless_module_attr_name
 
 # ....................{ IMPORTS                            }....................
+from beartype.meta import URL_ISSUES
 from beartype.roar import BeartypeDecorHintNonpepException
 from beartype._util.cache.utilcachecall import callable_cached
-from beartype._util.cls.pep.utilpep3119 import (
+from beartype._util.cls.pep.clspep3119 import (
     die_unless_type_isinstanceable,
-    is_type_or_types_isinstanceable,
+    is_type_isinstanceable,
 )
-from beartype._data.hint.datahinttyping import TypeException
+from beartype._data.typing.datatyping import TypeException
 
 # ....................{ VALIDATORS                         }....................
 #FIXME: Unit test us up, please.
@@ -33,7 +34,7 @@ def die_if_hint_nonpep(
     hint: object,
 
     # Optional parameters.
-    is_str_valid: bool = True,
+    is_forwardref_valid: bool = False,
     exception_cls: TypeException = BeartypeDecorHintNonpepException,
     exception_prefix: str = '',
 ) -> None:
@@ -49,24 +50,23 @@ def die_if_hint_nonpep(
     ----------
     hint : object
         Object to be validated.
-    is_str_valid : bool, optional
-        ``True`` only if this function permits this object to either be a
-        string or contain strings. Defaults to ``True``. If this boolean is:
+    is_forwardref_valid : bool, optional
+        :data:`True` only if this function permits this object to contain
+        forward references. Defaults to :data:`False`. If this boolean is:
 
-        * ``True``, this object is valid only if this object is either a class
-          or tuple of classes and/or classnames.
-        * ``False``, this object is valid only if this object is either a class
-          or tuple of classes.
+        * :data:`True`, this object is valid only when containing classes and/or
+          forward references.
+        * :data:`False`, this object is valid only when containing classes.
     exception_cls : type[Exception]
-        Type of the exception to be raised by this function. Defaults to
-        :class:`BeartypeDecorHintNonpepException`.
+        Type of exception to be raised. Defaults to
+        :exc:`.BeartypeDecorHintNonpepException`.
     exception_prefix : str, optional
-        Human-readable label prefixing the representation of this object in the
-        exception message. Defaults to the empty string.
+        Human-readable substring prefixing raised exception messages. Defaults
+        to the empty string.
 
     Raises
-    ----------
-    :exc:`exception_cls`
+    ------
+    exception_cls
         If this object is either:
 
         * An **isinstanceable type** (i.e., standard class passable as the
@@ -76,15 +76,14 @@ def die_if_hint_nonpep(
           or more:
 
           * Non-:mod:`typing` types.
-          * If ``is_str_valid``, **strings** (i.e., forward references
-            specified as either fully-qualified or unqualified classnames).
+          * If ``is_forwardref_valid`` is :data:`True`, forward references.
     '''
 
     # If this object is a PEP-noncompliant type hint, raise an exception.
     #
     # Note that this memoized call is intentionally passed positional rather
     # than keyword parameters to maximize efficiency.
-    if is_hint_nonpep(hint, is_str_valid):
+    if is_hint_nonpep(hint, is_forwardref_valid):
         assert isinstance(exception_prefix, str), (
             f'{repr(exception_prefix)} not string.')
         assert isinstance(exception_cls, type), (
@@ -94,14 +93,14 @@ def die_if_hint_nonpep(
 
         raise exception_cls(
             f'{exception_prefix}type hint {repr(hint)} '
-            f'is PEP-noncompliant (e.g., neither ' +
+            f'is PEP-noncompliant (e.g., either ' +
             (
                 (
-                    'isinstanceable class, forward reference, nor tuple of '
-                    'isinstanceable classes and/or forward references).'
+                    'isinstanceable type, forward reference, or tuple of '
+                    'isinstanceable types and/or forward references).'
                 )
-                if is_str_valid else
-                'isinstanceable class nor tuple of isinstanceable classes).'
+                if is_forwardref_valid else
+                'isinstanceable type or tuple of isinstanceable types).'
             )
         )
     # Else, this object is *NOT* a PEP-noncompliant type hint.
@@ -113,7 +112,7 @@ def die_unless_hint_nonpep(
     hint: object,
 
     # Optional parameters.
-    is_str_valid: bool = True,
+    is_forwardref_valid: bool = False,
     exception_cls: TypeException = BeartypeDecorHintNonpepException,
     exception_prefix: str = '',
 ) -> None:
@@ -122,31 +121,30 @@ def die_unless_hint_nonpep(
     hint** (i.e., :mod:`beartype`-specific annotation *not* compliant with
     annotation-centric PEPs).
 
-    This validator is effectively (but technically *not*) memoized. See the
+    This validator is effectively (but technically *not*) memoized. See also the
     :func:`beartype._util.hint.utilhinttest.die_unless_hint` validator.
 
     Parameters
     ----------
     hint : object
         Object to be validated.
-    is_str_valid : bool, optional
-        ``True`` only if this function permits this object to either be a
-        string or contain strings. Defaults to ``True``. If this boolean is:
+    is_forwardref_valid : bool, optional
+        :data:`True` only if this function permits this object to contain
+        forward references. Defaults to :data:`False`. If this boolean is:
 
-        * ``True``, this object is valid only if this object is either a class
-          or tuple of classes and/or classnames.
-        * ``False``, this object is valid only if this object is either a class
-          or tuple of classes.
+        * :data:`True`, this object is valid only when containing classes and/or
+          forward references.
+        * :data:`False`, this object is valid only when containing classes.
     exception_cls : type[Exception], optional
-        Type of the exception to be raised by this function. Defaults to
-        :class:`BeartypeDecorHintNonpepException`.
+        Type of exception to be raised. Defaults to
+        :exc:`.BeartypeDecorHintNonpepException`.
     exception_prefix : str, optional
-        Human-readable label prefixing the representation of this object in the
-        exception message. Defaults to the empty string.
+        Human-readable substring prefixing raised exception messages. Defaults
+        to the empty string.
 
     Raises
-    ----------
-    :exc:`exception_cls`
+    ------
+    exception_cls
         If this object is neither:
 
         * An **isinstanceable type** (i.e., standard class passable as the
@@ -156,15 +154,14 @@ def die_unless_hint_nonpep(
           or more:
 
           * Non-:mod:`typing` types.
-          * If ``is_str_valid``, **strings** (i.e., forward references
-            specified as either fully-qualified or unqualified classnames).
+          * If ``is_forwardref_valid`` is :data:`True`, forward references.
     '''
 
     # If this object is a PEP-noncompliant type hint, reduce to a noop.
     #
     # Note that this memoized call is intentionally passed positional rather
     # than keyword parameters to maximize efficiency.
-    if is_hint_nonpep(hint, is_str_valid):
+    if is_hint_nonpep(hint, is_forwardref_valid):
         return
     # Else, this object is *NOT* a PEP-noncompliant type hint. In this case,
     # subsequent logic raises an exception specific to the passed parameters.
@@ -177,57 +174,43 @@ def die_unless_hint_nonpep(
     assert isinstance(exception_prefix, str), (
         f'{repr(exception_prefix)} not string.')
 
-    # If this object is a class...
+    # If this object is a type...
     if isinstance(hint, type):
-        # If this class is *NOT* PEP-noncompliant, raise an exception.
+        # If this type is *NOT* PEP-noncompliant (e.g., is either PEP-compliant
+        # *OR* is not an isinstanceable type), raise an exception.
         die_unless_hint_nonpep_type(
             hint=hint,
+            is_forwardref_valid=is_forwardref_valid,
             exception_prefix=exception_prefix,
             exception_cls=exception_cls,
         )
+        # Else, this type is PEP-noncompliant and thus isinstanceable by
+        # definition.
 
-        # Else, this class is isinstanceable. In this case, silently accept
-        # this class as is.
+        # Silently accept this isinstanceable type as is.
         return
-    # Else, this object is *NOT* a class.
+    # Else, this object is *NOT* a type.
     #
     # If this object is a tuple, raise a tuple-specific exception.
     elif isinstance(hint, tuple):
         die_unless_hint_nonpep_tuple(
             hint=hint,
+            is_forwardref_valid=is_forwardref_valid,
             exception_prefix=exception_prefix,
-            is_str_valid=is_str_valid,
             exception_cls=exception_cls,
         )
-    # Else, this object is neither a type nor type tuple.
+    # Else, this object is neither a type *NOR* tuple.
 
-    # Raise a generic exception.
-    raise exception_cls(
-        f'{exception_prefix}type hint {repr(hint)} either '
-        f'PEP-noncompliant or currently unsupported by @beartype.'
+    # Avoid circular import dependencies.
+    from beartype._util.hint.utilhinttest import die_as_hint_unsupported
+
+    # Raise a generic exception message as a fallback.
+    die_as_hint_unsupported(
+        hint=hint,
+        exception_prefix=exception_prefix,
+        exception_cls=exception_cls,
     )
 
-    #FIXME: Temporarily preserved in case we want to restore this. *shrug*
-    # # Else, this object is neither a forward reference, class, nor tuple. Ergo,
-    # # this object is *NOT* a PEP-noncompliant type hint.
-    # #
-    # # If forward references are supported, raise an exception noting that.
-    # elif is_str_valid:
-    #     raise exception_cls(
-    #         f'{exception_prefix}type hint {repr(hint)} '
-    #         f'neither PEP-compliant nor -noncompliant '
-    #         f'(e.g., neither PEP-compliant, isinstanceable class, forward reference, or '
-    #         f'tuple of isinstanceable classes and forward references).'
-    #     )
-    # # Else, forward references are unsupported. In this case, raise an
-    # # exception noting that.
-    # else:
-    #     raise exception_cls(
-    #         f'{exception_prefix}type hint {repr(hint)} '
-    #         f'neither PEP-compliant nor -noncompliant '
-    #         f'(e.g., isinstanceable class or tuple of isinstanceable classes).'
-    #     )
-
 # ....................{ VALIDATORS ~ kind                  }....................
 #FIXME: Unit test us up.
 def die_unless_hint_nonpep_type(
@@ -235,6 +218,7 @@ def die_unless_hint_nonpep_type(
     hint: type,
 
     # Optional parameters.
+    is_forwardref_valid: bool = False,
     exception_cls: TypeException = BeartypeDecorHintNonpepException,
     exception_prefix: str = '',
 ) -> None:
@@ -251,19 +235,26 @@ def die_unless_hint_nonpep_type(
     ----------
     hint : type
         Object to be validated.
+    is_forwardref_valid : bool, optional
+        :data:`True` only if this function permits this object to be a
+        forward reference. Defaults to :data:`False`. If this boolean is:
+
+        * :data:`True`, this object is valid only when a class and/or forward
+          reference.
+        * :data:`False`, this object is valid only when a class.
     exception_cls : Optional[type]
         Type of the exception to be raised by this function. Defaults to
-        :class:`BeartypeDecorHintNonpepException`.
+        :exc:`.BeartypeDecorHintNonpepException`.
     exception_prefix : str, optional
         Human-readable label prefixing the representation of this object in the
         exception message. Defaults to the empty string.
 
     Raises
-    ----------
+    ------
     BeartypeDecorHintPep3119Exception
         If this object is *not* an isinstanceable class (i.e., class passable
         as the second argument to the :func:`isinstance` builtin).
-    :exc:`exception_cls`
+    exception_cls
         If this object is a PEP-compliant type hint.
     '''
 
@@ -276,13 +267,14 @@ def die_unless_hint_nonpep_type(
         exception_cls=exception_cls,
         exception_prefix=exception_prefix,
     )
-    # Else, this object is *NOT* a PEP-noncompliant type hint.
-    #
+    # Else, this object is *NOT* a PEP-compliant type hint.
+
     # If this object is *NOT* an isinstanceable class, raise an exception. Note
     # that this validation is typically slower than the prior validation and
     # thus intentionally performed last.
     die_unless_type_isinstanceable(
         cls=hint,
+        is_forwardref_valid=is_forwardref_valid,
         exception_cls=exception_cls,
         exception_prefix=exception_prefix,
     )
@@ -297,25 +289,25 @@ def die_unless_hint_nonpep_type(
 #internally raise a stop iteration exception, whereas EAFP only raises an
 #exception if this tuple is invalid, in which case efficiency is no longer a
 #concern. So, what do we do instead? Simple. We internally refactor:
-#* If "is_str_valid" is True, we continue to perform the existing
+#* If "is_forwardref_valid" is True, we continue to perform the existing
 #  implementation of both functions. *shrug*
 #* Else, we:
 #  * Perform a new optimized EAFP-style isinstance() check resembling that
 #    performed by die_unless_type_isinstanceable().
-#  * Likewise for _is_hint_nonpep_tuple() vis-a-vis is_type_or_types_isinstanceable().
+#  * Likewise for _is_hint_nonpep_tuple() vis-a-vis is_type_isinstanceable().
 #Fortunately, tuple unions are now sufficiently rare in the wild (i.e., in
 #real-world use cases) that this mild inefficiency probably no longer matters.
-#FIXME: Indeed! Now that we have the die_unless_type_or_types_isinstanceable()
+#FIXME: Indeed! Now that we have the die_unless_object_isinstanceable()
 #validator, this validator should reduce to efficiently calling
-#die_unless_type_or_types_isinstanceable() directly if "is_str_valid" is False.
-#die_unless_type_or_types_isinstanceable() performs the desired EAFP-style
+#die_unless_object_isinstanceable() directly if "is_forwardref_valid" is False.
+#die_unless_object_isinstanceable() performs the desired EAFP-style
 #isinstance() check in an optimally efficient manner.
 def die_unless_hint_nonpep_tuple(
     # Mandatory parameters.
     hint: object,
 
     # Optional parameters.
-    is_str_valid: bool = False,
+    is_forwardref_valid: bool = False,
     exception_cls: TypeException = BeartypeDecorHintNonpepException,
     exception_prefix: str = '',
 ) -> None:
@@ -331,23 +323,23 @@ def die_unless_hint_nonpep_tuple(
     ----------
     hint : object
         Object to be validated.
-    is_str_valid : bool, optional
-        ``True`` only if this function permits this tuple to contain strings.
-        Defaults to ``False``. If:
+    is_forwardref_valid : bool, optional
+        :data:`True` only if this function permits this tuple to contain
+        forward references. Defaults to :data:`False`. If this boolean is:
 
-        * ``True``, this tuple is valid only when containing classes and/or
-          classnames.
-        * ``False``, this tuple is valid only when containing classes.
+        * :data:`True`, this tuple is valid only when containing classes and/or
+          forward references.
+        * :data:`False`, this tuple is valid only when containing classes.
     exception_cls : type, optional
         Type of the exception to be raised by this function. Defaults to
-        :class:`BeartypeDecorHintNonpepException`.
+        :exc:`.BeartypeDecorHintNonpepException`.
     exception_prefix : str, optional
         Human-readable label prefixing the representation of this object in the
         exception message. Defaults to the empty string.
 
     Raises
-    ----------
-    :exc:`exception_cls`
+    ------
+    exception_cls
         If this object is neither:
 
         * A non-:mod:`typing` type (i.e., class *not* defined by the
@@ -359,20 +351,21 @@ def die_unless_hint_nonpep_tuple(
           or more:
 
           * Non-:mod:`typing` types.
-          * If ``is_str_valid``, **strings** (i.e., forward references
-            specified as either fully-qualified or unqualified classnames).
+          * If ``is_forwardref_valid`` is :data:`True`, forward references.
     '''
 
     # If this object is a tuple union, reduce to a noop.
     #
     # Note that this memoized call is intentionally passed positional rather
     # than keyword parameters to maximize efficiency.
-    if _is_hint_nonpep_tuple(hint, is_str_valid):
+    if _is_hint_nonpep_tuple(hint, is_forwardref_valid):
         return
     # Else, this object is *NOT* a tuple union. In this case, subsequent logic
     # raises an exception specific to the passed parameters.
     #
-    # Note that the prior call has already validated "is_str_valid".
+    # Note that the prior call has already validated "is_forwardref_valid".
+    assert isinstance(is_forwardref_valid, bool), (
+        f'{repr(is_forwardref_valid)} not bool.')
     assert isinstance(exception_cls, type), f'{repr(exception_cls)} not type.'
     assert isinstance(exception_prefix, str), (
         f'{repr(exception_prefix)} not string.')
@@ -403,15 +396,17 @@ def die_unless_hint_nonpep_tuple(
             # If this class is *NOT* isinstanceable, raise an exception.
             die_unless_type_isinstanceable(
                 cls=hint_item,
+                is_forwardref_valid=is_forwardref_valid,
                 exception_prefix=exception_prefix,
                 exception_cls=exception_cls,
             )
+            # Else, this class is isinstanceable.
         # Else, this item is *NOT* a class.
         #
         # If this item is a forward reference...
         elif isinstance(hint_item, str):
             # If forward references are unsupported, raise an exception.
-            if not is_str_valid:
+            if not is_forwardref_valid:
                 raise exception_cls(
                     f'{exception_prefix}tuple type hint {repr(hint)} '
                     f'forward reference "{hint_item}" unsupported.'
@@ -425,7 +420,7 @@ def die_unless_hint_nonpep_tuple(
             raise exception_cls(
                 f'{exception_prefix}tuple type hint {repr(hint)} '
                 f'item {repr(hint_item)} invalid '
-                f'{"neither type nor string" if is_str_valid else "not type"}.'
+                f'{"neither type nor string" if is_forwardref_valid else "not type"}.'
             )
 
 # ....................{ TESTERS                            }....................
@@ -434,10 +429,10 @@ def is_hint_nonpep(
     hint: object,
 
     # Optional parameters.
-    is_str_valid: bool = False,
+    is_forwardref_valid: bool = False,
 ) -> bool:
     '''
-    ``True`` only if the passed object is a **PEP-noncompliant type hint**
+    :data:`True` only if the passed object is a **PEP-noncompliant type hint**
     (i.e., :mod:`beartype`-specific annotation *not* compliant with
     annotation-centric PEPs).
 
@@ -449,19 +444,20 @@ def is_hint_nonpep(
     ----------
     hint : object
         Object to be inspected.
-    is_str_valid : bool, optional
-        ``True`` only if this function permits this object to be a string.
-        Defaults to ``False``. If this boolean is:
+    is_forwardref_valid : bool, optional
+        :data:`True` only if this function permits this object to contain
+        forward references. If this boolean is:
 
-        * ``True``, this object is valid only if this object is either a class
-          or tuple of classes and/or classnames.
-        * ``False``, this object is valid only if this object is either a class
-          or tuple of classes.
+        * :data:`True`, this object is valid only when containing classes and/or
+          forward references.
+        * :data:`False`, this object is valid only when containing classes.
+
+        Defaults to :data:`False` for safety.
 
     Returns
-    ----------
+    -------
     bool
-        ``True`` only if this object is either:
+        :data:`True` only if this object is either:
 
         * A non-:mod:`typing` type (i.e., class *not* defined by the
           :mod:`typing` module, whose public classes are used to instantiate
@@ -472,10 +468,10 @@ def is_hint_nonpep(
           or more:
 
           * Non-:mod:`typing` types.
-          * If ``is_str_valid``, **strings** (i.e., forward references
-            specified as either fully-qualified or unqualified classnames).
+          * If ``is_forwardref_valid`` is :data:`True`, forward references.
     '''
-    assert isinstance(is_str_valid, bool), f'{repr(is_str_valid)} not boolean.'
+    assert isinstance(is_forwardref_valid, bool), (
+        f'{repr(is_forwardref_valid)} not bool.')
 
     #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     # BEGIN: Synchronize changes here with die_unless_hint_nonpep() above.
@@ -483,84 +479,36 @@ def is_hint_nonpep(
 
     # Return true only if either...
     return (
-        # If this object is a class, return true only if this is *NOT* a
-        # PEP-compliant class, in which case this *MUST* be a PEP-noncompliant
-        # class by definition.
-        _is_hint_nonpep_type(hint) if isinstance(hint, type) else
+        # If this object is a class, return true only if this is is a
+        # PEP-noncompliant class (or possibly a caller-permitted forward
+        # reference).
+        is_hint_nonpep_type(
+            hint, is_forwardref_valid) if isinstance(hint, type) else
         # Else, this object is *NOT* a class.
         #
         # If this object is a tuple, return true only if this tuple contains
-        # only one or more caller-permitted forward references and
-        # PEP-noncompliant classes.
-        _is_hint_nonpep_tuple(hint, is_str_valid) if isinstance(hint, tuple)
-        # Else, this object is neither a class nor tuple. Return false, as this
-        # object *CANNOT* be PEP-noncompliant.
-        else False
+        # only one or more PEP-noncompliant classes (and possibly
+        # caller-permitted forward references).
+        _is_hint_nonpep_tuple(
+            hint, is_forwardref_valid) if isinstance(hint, tuple) else
+        # Else, this object is neither a class nor tuple and thus *CANNOT* be
+        # PEP-noncompliant. In this case, fallback to returning false.
+        False
     )
 
-# ....................{ TESTERS ~ private                  }....................
-@callable_cached
-def _is_hint_nonpep_tuple(
+
+#FIXME: Unit test us up, please.
+def is_hint_nonpep_type(
     # Mandatory parameters.
     hint: object,
 
     # Optional parameters.
-    is_str_valid: bool = False,
+    is_forwardref_valid: bool = False,
 ) -> bool:
     '''
-    ``True`` only if the passed object is a PEP-noncompliant non-empty tuple of
-    one or more types.
-
-    This tester is memoized for efficiency.
-
-    Parameters
-    ----------
-    hint : object
-        Object to be inspected.
-    is_str_valid : bool, optional
-        ``True`` only if this function permits this tuple to contain strings.
-        Defaults to ``False``. If this boolean is:
-
-        * ``True``, this tuple is valid only when containing classes and/or
-          classnames.
-        * ``False``, this object is valid only when containing classes.
-
-    Returns
-    ----------
-    bool
-        ``True`` only if this object is a **non-empty tuple** (i.e., semantic
-        union of types) containing one or more:
-
-          * Non-:mod:`typing` types.
-          * If ``is_str_valid``, **strings** (i.e., forward references
-            specified as either fully-qualified or unqualified classnames).
-    '''
-    assert isinstance(is_str_valid, bool), f'{repr(is_str_valid)} not boolean.'
-
-    #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-    # BEGIN: Synchronize changes here with die_unless_hint_nonpep() above.
-    #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-
-    # Return true only if this object is...
-    return (
-        # A tuple *AND*...
-        isinstance(hint, tuple) and
-        # This tuple is non-empty *AND*...
-        len(hint) > 0 and
-        # Each item of this tuple is either a caller-permitted forward
-        # reference *OR* an isinstanceable class.
-        all(
-            is_type_or_types_isinstanceable(hint_item) if isinstance(hint_item, type) else
-            is_str_valid                               if isinstance(hint_item, str) else
-            False
-            for hint_item in hint
-        )
-    )
-
-
-def _is_hint_nonpep_type(hint: object) -> bool:
-    '''
-    ``True`` only if the passed object is a PEP-noncompliant type.
+    :data:`True` only if the passed object is a PEP-noncompliant isinstanceable
+    type (and/or forward reference proxy when ``is_forwardref_valid`` is
+    :data:`True`).
 
     This tester is intentionally *not* memoized (e.g., by the
     :func:`callable_cached` decorator), as the implementation trivially reduces
@@ -570,12 +518,35 @@ def _is_hint_nonpep_type(hint: object) -> bool:
     ----------
     hint : object
         Object to be inspected.
+    is_forwardref_valid : bool, optional
+        :data:`True` only if this function permits this object to be a
+        **forward reference proxy** (i.e., :mod:`beartype`-specific private type
+        proxying an external type that may currently be undefined). If this
+        boolean is:
+
+        * :data:`True`, this object is valid only when this object is either:
+
+          * An isinstanceable type.
+          * A forward reference proxy (regardless of whether this proxy is
+            currently resolvable to an isinstanceable type that has already been
+            externally defined).
+
+        * :data:`False`, this object is valid only when this object is either:
+
+          * An isinstanceable type.
+          * A forward reference proxy that is currently resolvable to an
+            isinstanceable type that has already been externally defined.
+
+        Defaults to :data:`False` for safety.
 
     Returns
-    ----------
+    -------
     bool
-        ``True`` only if this object is a PEP-noncompliant type.
+        :data:`True` only if this object is a PEP-noncompliant isinstanceable
+        type (and/or forward reference proxy when ``is_forwardref_valid`` is
+        :data:`True`).
     '''
+
     #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     # BEGIN: Synchronize changes here with die_unless_hint_nonpep() above.
     #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
@@ -583,7 +554,90 @@ def _is_hint_nonpep_type(hint: object) -> bool:
     # Avoid circular import dependencies.
     from beartype._util.hint.pep.utilpeptest import is_hint_pep
 
-    # Return true only if this object is isinstanceable and *NOT* a
-    # PEP-compliant class, in which case this *MUST* be a PEP-noncompliant
-    # class by definition.
-    return is_type_or_types_isinstanceable(hint) and not is_hint_pep(hint)
+    # Return true only if...
+    return (
+        # This object is an isinstanceable type *AND*...
+        is_type_isinstanceable(hint, is_forwardref_valid) and
+        # This object is *NOT* a PEP-compliant type, in which case this object
+        # is a PEP-noncompliant type by definition.
+        not is_hint_pep(hint)
+    )
+
+# ....................{ TESTERS ~ private                  }....................
+@callable_cached
+def _is_hint_nonpep_tuple(hint: object, is_forwardref_valid: bool) -> bool:
+    '''
+    :data:`True` only if the passed object is a PEP-noncompliant non-empty tuple
+    of one or more types.
+
+    This tester is memoized for efficiency.
+
+    Parameters
+    ----------
+    hint : object
+        Object to be inspected.
+    is_forwardref_valid : bool, optional
+        :data:`True` only if this function permits this tuple to contain
+        forward references. If this boolean is:
+
+        * :data:`True`, this tuple is valid only when containing classes and/or
+          classnames.
+        * :data:`False`, this tuple is valid only when containing classes.
+
+    Returns
+    -------
+    bool
+        :data:`True` only if this object is a **non-empty tuple** (i.e.,
+        semantic union of types) containing one or more:
+
+          * Non-:mod:`typing` types.
+          * If ``is_forwardref_valid`` is :data:`True`, forward references.
+    '''
+    assert isinstance(is_forwardref_valid, bool), (
+        f'{repr(is_forwardref_valid)} not bool.')
+
+    #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+    # BEGIN: Synchronize changes here with die_unless_hint_nonpep() above.
+    #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+    # If it is *NOT* the case that this object is...
+    if not (
+        # A tuple *AND*...
+        isinstance(hint, tuple) and
+        # This tuple is non-empty *AND*...
+        hint
+    ):
+        # This object is either not a tuple *OR* is the empty tuple. In either
+        # case, this object is *NOT* a non-empty tuple. Return false.
+        return False
+    # Else, this object is a non-empty tuple.
+
+    # For each item of this tuple...
+    for hint_item in hint:
+        # print(f'Inspecting {repr(hint)} item {repr(hint_item)}...')
+
+        # If this item is a type...
+        if isinstance(hint_item, type):
+            # If this type is *NOT* isinstanceable, return false.
+            if not is_hint_nonpep_type(hint_item, is_forwardref_valid):
+                # print(f'Non-isinstanceable type {repr(hint_item)} prohibited!')
+                return False
+            # Else, this type is isinstanceable.
+        # Else, this item is *NOT* a type.
+        #
+        # If this item is a string...
+        elif isinstance(hint_item, str):
+            # If the caller prohibits forward references, return false.
+            if not is_forwardref_valid:
+                # print(f'Forward reference {repr(hint_item)} prohibited!')
+                return False
+            # Else, the caller permits forward references.
+        # Else, this item is *NOT* a string. In this case, this item is
+        # prohibited as a tuple item of a PEP-noncompliant tuple. Return false.
+        else:
+            # print(f'Unexpected item {repr(hint_item)} prohibited!')
+            return False
+    # Else, all items of this tuple are permitted as tuple items of a
+    # PEP-noncompliant tuple.
+
+    # Return true.
+    return True
