@@ -4,8 +4,9 @@
 # See "LICENSE" for further details.
 
 '''
-Test-specific **test suite paths** (i.e., :class:`pathlib.Path` instances
-encapsulating test-specific paths unique to this test suite).
+Test-wide **test suite paths** (i.e., low-level callables creating and returning
+:class:`pathlib.Path` objects encapsulating test-specific paths unique to this
+test suite).
 '''
 
 # ....................{ IMPORTS                            }....................
@@ -25,7 +26,7 @@ from pathlib import Path
 @callable_cached
 def get_test_package_dir() -> Path:
     '''
-    :mod:`Path` encapsulating the absolute dirname of the **top-level test
+    :class:`.Path` encapsulating the absolute dirname of the **top-level test
     package** (i.e., directory providing this project's top-level test package
     containing at least an ``__init__.py`` file) if found *or* raise an
     exception otherwise.
@@ -41,8 +42,8 @@ def get_test_package_dir() -> Path:
 @callable_cached
 def get_test_func_subpackage_dir() -> Path:
     '''
-    :mod:`Path` encapsulating the absolute dirname of the **mid-level
-    functional test subpackage** (i.e., directory providing all functional
+    :class:`.Path` encapsulating the absolute dirname of the **mid-level
+    integration test subpackage** (i.e., directory providing all integration
     tests of this project's test suite) if found *or* raise an exception
     otherwise.
     '''
@@ -54,22 +55,22 @@ def get_test_func_subpackage_dir() -> Path:
 @callable_cached
 def get_test_func_data_dir() -> Path:
     '''
-    :mod:`Path` encapsulating the absolute dirname of the **mid-level
-    functional test data directory** (i.e., directory providing sample data
-    used throughout this project's functional tests) if found *or* raise an
+    :class:`.Path` encapsulating the absolute dirname of the **mid-level
+    integration test data directory** (i.e., directory providing sample data
+    used throughout this project's integration tests) if found *or* raise an
     exception otherwise.
     '''
 
     # Questionable destination!
     return DirRelative(get_test_func_subpackage_dir(), 'data')
 
-# ....................{ GETTERS ~ dir : func : api         }....................
+# ....................{ GETTERS ~ dir : func : lib         }....................
 @callable_cached
 def get_test_func_data_lib_dir() -> Path:
     '''
-    :mod:`Path` encapsulating the absolute dirname of the **mid-level
-    third-party dependency functional test data directory** (i.e., directory
-    providing sample data used throughout this project's functional tests
+    :class:`.Path` encapsulating the absolute dirname of the **mid-level
+    third-party dependency integration test data directory** (i.e., directory
+    providing sample data used throughout this project's integration tests
     exercising third-party dependencies) if found *or* raise an exception
     otherwise.
     '''
@@ -81,9 +82,9 @@ def get_test_func_data_lib_dir() -> Path:
 @callable_cached
 def get_test_func_data_lib_nuitka_dir() -> Path:
     '''
-    :mod:`Path` encapsulating the absolute dirname of the **low-level nuitka
-    functional test data directory** (i.e., directory providing sample code
-    used throughout this project's :mod:`nuitka`-specific functional tests) if
+    :class:`.Path` encapsulating the absolute dirname of the **low-level nuitka
+    integration test data directory** (i.e., directory providing sample code
+    used throughout this project's :mod:`nuitka`-specific integration tests) if
     found *or* raise an exception otherwise.
     '''
 
@@ -91,12 +92,25 @@ def get_test_func_data_lib_nuitka_dir() -> Path:
     return DirRelative(get_test_func_data_lib_dir(), 'nuitka')
 
 
+@callable_cached
+def get_test_func_data_lib_poetry_dir() -> Path:
+    '''
+    :class:`.Path` encapsulating the absolute dirname of the **low-level Poetry
+    integration test data directory** (i.e., directory providing sample code
+    used throughout this project's :mod:`poetry`-specific integration tests) if
+    found *or* raise an exception otherwise.
+    '''
+
+    # Nascent ascendency!
+    return DirRelative(get_test_func_data_lib_dir(), 'poetry')
+
+
 @callable_cached
 def get_test_func_data_lib_sphinx_dir() -> Path:
     '''
-    :mod:`Path` encapsulating the absolute dirname of the **low-level Sphinx
-    functional test data directory** (i.e., directory providing sample data
-    used throughout this project's :mod:`sphinx`-specific functional tests) if
+    :class:`.Path` encapsulating the absolute dirname of the **low-level Sphinx
+    integration test data directory** (i.e., directory providing sample data
+    used throughout this project's :mod:`sphinx`-specific integration tests) if
     found *or* raise an exception otherwise.
     '''
 
@@ -107,9 +121,9 @@ def get_test_func_data_lib_sphinx_dir() -> Path:
 @callable_cached
 def get_test_func_data_lib_nuitka_file() -> Path:
     '''
-    :mod:`Path` encapsulating the absolute filename of the **low-level nuitka
-    functional test data file** (i.e., file providing sample code used
-    throughout this project's :mod:`nuitka`-specific functional tests) if found
+    :class:`.Path` encapsulating the absolute filename of the **low-level nuitka
+    integration test data file** (i.e., file providing sample code used
+    throughout this project's :mod:`nuitka`-specific integration tests) if found
     *or* raise an exception otherwise.
     '''
 
@@ -121,9 +135,9 @@ def get_test_func_data_lib_nuitka_file() -> Path:
 @callable_cached
 def get_test_unit_subpackage_dir() -> Path:
     '''
-    :mod:`Path` encapsulating the absolute dirname of the **mid-level unit test
-    subpackage** (i.e., directory providing all unit tests of this project's
-    test suite) if found *or* raise an exception otherwise.
+    :class:`.Path` encapsulating the absolute dirname of the **mid-level unit
+    test subpackage** (i.e., directory providing all unit tests of this
+    project's test suite) if found *or* raise an exception otherwise.
     '''
 
     # Redacted didactic!
@@ -133,9 +147,9 @@ def get_test_unit_subpackage_dir() -> Path:
 @callable_cached
 def get_test_unit_data_dir() -> Path:
     '''
-    :mod:`Path` encapsulating the absolute dirname of the **mid-level unit test
-    data directory** (i.e., directory providing sample data used throughout this
-    project's unit tests) if found *or* raise an exception otherwise.
+    :class:`.Path` encapsulating the absolute dirname of the **mid-level unit
+    test data directory** (i.e., directory providing sample data used throughout
+    this project's unit tests) if found *or* raise an exception otherwise.
     '''
 
     # Galactic antacid!
@@ -145,7 +159,7 @@ def get_test_unit_data_dir() -> Path:
 @callable_cached
 def get_test_unit_data_claw_dir() -> Path:
     '''
-    :mod:`Path` encapsulating the absolute dirname of the **mid-level import
+    :class:`.Path` encapsulating the absolute dirname of the **mid-level import
     hook unit test data directory** (i.e., directory providing sample data used
     throughout this project's unit tests exercising import hooks published by
     the :mod:`beartype.claw` subpackage) if found *or* raise an exception
@@ -159,7 +173,7 @@ def get_test_unit_data_claw_dir() -> Path:
 @callable_cached
 def get_test_unit_data_claw_extraprocess_dir() -> Path:
     '''
-    :mod:`Path` encapsulating the absolute dirname of the **mid-level
+    :class:`.Path` encapsulating the absolute dirname of the **mid-level
     extraprocess import hook unit test data directory** (i.e., directory
     providing sample data used throughout this project's unit tests exercising
     import hooks published by the :mod:`beartype.claw` subpackage in Python
@@ -174,7 +188,7 @@ def get_test_unit_data_claw_extraprocess_dir() -> Path:
 @callable_cached
 def get_test_unit_data_door_dir() -> Path:
     '''
-    :mod:`Path` encapsulating the absolute dirname of the **mid-level Decidedly
+    :class:`.Path` encapsulating the absolute dirname of the **mid-level Decidedly
     Object-Oriented Runtime-checking (DOOR) unit test data directory** (i.e.,
     directory providing sample data used throughout this project's unit tests
     exercising type-checkers published by the :mod:`beartype.door` subpackage)
@@ -188,7 +202,7 @@ def get_test_unit_data_door_dir() -> Path:
 @callable_cached
 def get_test_unit_data_door_extraprocess_dir() -> Path:
     '''
-    :mod:`Path` encapsulating the absolute dirname of the **mid-level
+    :class:`.Path` encapsulating the absolute dirname of the **mid-level
     extraprocess Decidedly Object-Oriented Runtime-checking (DOOR) unit test
     data directory** (i.e., directory providing sample data used throughout this
     project's unit tests exercising type-checkers published by the
